# -*- coding: utf-8 -*-

"""
examples.finalizers
~~~~~~~~~~~~~~~~~~~

Success and Failure finalizers.
"""

import time

from yaspin import yaspin


def default_finalizers():
    with yaspin(text="Downloading...") as sp:
        time.sleep(2)
        sp.ok()

    with yaspin(text="Downloading...") as sp:
        time.sleep(2)
        sp.fail()


def custom_finalizers():
    with yaspin(text="Processing...") as sp:
        time.sleep(2)
        sp.ok('☀️')

    with yaspin(text="Processing...", right=True) as sp:
        time.sleep(2)
        sp.fail('💥')

    with yaspin(text="Processing...") as sp:
        time.sleep(2)
        sp.ok(b'\xe2\x9c\x94')


def main():
    default_finalizers()
    custom_finalizers()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""
tests.test_properties
~~~~~~~~~~~~~~~~~~~~~

Test getters and setters.
"""

from inspect import getsource

import pytest

from yaspin import Spinner, yaspin
from yaspin.base_spinner import default_spinner
from yaspin.compat import basestring, builtin_str, str
from yaspin.helpers import to_unicode


#
# Yaspin.spinner
#
def test_spinner_getter(frames, interval):
    swirl = yaspin()
    assert swirl.spinner == default_spinner

    new_spinner = Spinner(frames, interval)
    swirl.spinner = new_spinner
    assert swirl.spinner == swirl._set_spinner(new_spinner)


def test_spinner_setter(frames, interval):
    swirl = yaspin()
    assert swirl._spinner == default_spinner
    assert isinstance(swirl._frames, str)
    assert swirl._interval == swirl._spinner.interval * 0.001
    assert isinstance(repr(swirl), builtin_str)

    new_spinner = Spinner(frames, interval)
    swirl.spinner = new_spinner
    assert swirl._spinner == swirl._set_spinner(new_spinner)

    if isinstance(swirl._frames, basestring):
        assert isinstance(swirl._frames, str)

    if isinstance(swirl._frames, (list, tuple)):
        assert isinstance(swirl._frames[0], str)

    assert swirl._interval == swirl._spinner.interval * 0.001
    assert isinstance(repr(swirl), builtin_str)


#
# Yaspin.text
#
def test_text_getter(text):
    swirl = yaspin(text=text)
    assert swirl.text == to_unicode(text)


def test_text_setter(text):
    swirl = yaspin()
    swirl.text = text
    assert isinstance(swirl._text, str)
    assert swirl._text == to_unicode(text)


#
# Yaspin.right
#
def test_right_getter(right):
    swirl = yaspin(right=right)
    assert swirl.right == right


def test_right_setter(right):
    swirl = yaspin()
    swirl.right = right
    assert swirl._right == right


#
# Yaspin.reverse
#
def test_reverse_getter(reverse):
    swirl = yaspin(reverse=reverse)
    assert swirl.reverse == reverse


def test_reverse_setter(reverse):
    swirl = yaspin()
    swirl.reverse = reverse
    assert isinstance(swirl._frames, str)
    assert swirl._reverse == reverse


#
# Yaspin.color
#
def test_color_getter(supported_colors):
    color = supported_colors
    swirl = yaspin(color=color)
    assert swirl.color == color


def test_color_setter(colors_test_cases):
    color, expected = colors_test_cases
    swirl = yaspin()

    # Exception
    if isinstance(expected, Exception):
        with pytest.raises(type(expected)):
            swirl.color = color

    # Callable arg
    elif callable(color):
        # Compare source code to check funcs equality
        swirl.color = color
        fn1 = swirl._color
        fn2 = expected
        assert getsource(fn1) == getsource(fn2)

    # Common arg
    else:
        swirl.color = color
        assert swirl._color == expected



# -*- coding: utf-8 -*-

"""
tests.test_in_out
~~~~~~~~~~~~~~~~~

Checks that all input data is converted to unicode.
And all output data is converted to builtin str type.
"""

import sys

import pytest

from yaspin import Spinner, yaspin
from yaspin.compat import PY2, basestring, builtin_str, str
from yaspin.constants import ENCODING
from yaspin.helpers import to_unicode


def test_input_converted_to_unicode(text, frames, interval, right, reverse):
    sp = Spinner(frames, interval)
    swirl = yaspin(sp, text, right=right, reverse=reverse)

    if isinstance(swirl._frames, basestring):
        assert isinstance(swirl._frames, str)

    if isinstance(swirl._frames, (list, tuple)):
        assert isinstance(swirl._frames[0], str)

    assert isinstance(swirl._text, str)


def test_out_converted_to_builtin_str(text, frames, interval, right, reverse):
    sp = Spinner(frames, interval)
    swirl = yaspin(sp, text, right=right, reverse=reverse)

    for _ in range(len(frames)):
        frame = next(swirl._cycle)
        out = swirl._compose_out(frame)
        assert isinstance(out, builtin_str)


def test_repr(text, frames, interval):
    sp = Spinner(frames, interval)
    swirl = yaspin(sp, text)

    assert isinstance(repr(swirl), builtin_str)


def test_compose_out_with_color(colors_test_cases):
    color, expected = colors_test_cases

    # Skip non relevant cases
    empty = not expected
    is_exc = isinstance(expected, Exception)
    func = callable(expected)

    if empty or is_exc or func:
        pytest.skip("{0} - unsupported case".format(repr(expected)))

    # Actual test
    swirl = yaspin(color=color)
    out = swirl._compose_out(frame=u'/')
    assert out.startswith('\r\033')
    assert isinstance(out, builtin_str)


def test_write(capsys, text):
    swirl = yaspin()
    swirl.write(text)

    out, _ = capsys.readouterr()
    # cleans stdout from _clear_line and \r
    out = out.replace('\r\033[K', '')

    # handle out and text encodings (come out messy in PY2)
    # Under PY2 ``capsys.readouterr`` always produces ``out``
    # of type ``unicode``. Conversion to bytes is required
    # for proper ``out`` and ``text`` comparison.
    if PY2:
        out = out.encode(ENCODING)
        if isinstance(text, str):
            text = text.encode(ENCODING)

    assert isinstance(out, basestring)
    assert out[-1] == '\n'
    if text:
        assert out[:-1] == text


def test_hide_show(capsys, text, request):
    # Setup
    swirl = yaspin()
    swirl.start()

    # Ensure that swirl.stop() will be executed
    def teardown():
        swirl.stop()
    request.addfinalizer(teardown)

    #
    # Actual test
    #
    swirl.hide()

    # ensure that hidden spinner flag was set
    assert swirl._hide_spin.is_set()
    out, _ = capsys.readouterr()

    # ensure that text was cleared with the hide method
    assert out[-4:] == '\r\033[K'

    # properly encode text to unicode if running in PY2
    if PY2:
        text = to_unicode(text).encode(ENCODING)

    # ``\n`` is required to flush stdout during
    # the hidden state of the spinner
    sys.stdout.write('{0}\n'.format(text))
    out, _ = capsys.readouterr()

    # cleans stdout from _clear_line and \r
    out = out.replace('\r\033[K', '')

    # handle out and text encodings (come out messy in PY2)
    # Under PY2 ``capsys.readouterr`` always produces ``out``
    # of type ``unicode``. Conversion to bytes is required
    # for proper ``out`` and ``text`` comparison.
    if PY2:
        out = out.encode(ENCODING)
        if isinstance(text, str):
            text = text.encode(ENCODING)

    assert isinstance(out, basestring)
    assert out[-1] == '\n'
    if text:
        assert out[:-1] == text

    swirl.show()

    # ensure that hidden spinner flag was cleared
    assert not swirl._hide_spin.is_set()
    out, _ = capsys.readouterr()

    # ensure that text was cleared before resuming the spinner
    assert out[:4] == '\r\033[K'



# -*- coding: utf-8 -*-

"""
tests.conftest
~~~~~~~~~~~~~~

Tests data.
"""

import pytest

from yaspin.termcolor import colored


frame_cases = [
    # XXX: try byte strings
    # String types
    "",                                            # empty
    "+x*",                                         # ascii str
    "⢄⢂⢁⡁⡈⡐⡠",                                     # non-ascii in str
    u"⢹⢺⢼⣸",                                       # non-ascii in unicode str

    # Lists
    [],                                            # List[]
    [b"\xf0\x9f\x8c\xb2", b"\xf0\x9f\x8e\x84"],    # List[bytes]
    [u"🌲", u"🎄"],                                # List[unicode]
    ["⢹", "⢺", "⢼", "⣸"],                          # List[str], non-ascii

    # Tuples
    (),                                            # Tuple[]
    (b"\xf0\x9f\x8c\xb2", b"\xf0\x9f\x8e\x84"),    # Tuple[bytes]
    (u"🌲", u"🎄"),                                # Tuple[unicode]
    ("⢹", "⢺", "⢼", "⣸"),                          # Tuple[str], non-ascii
]


frame_ids = [
    # String types
    "'empty str'",
    "'ascii str'",
    "'non-ascii str'",
    "'non-ascii unicode str'",

    # Lists
    "'List[]'",
    "'List[bytes]'",
    "'List[unicode]'",
    "'List[str] non-ascii'",

    # Tuples
    "'Tuple[]'",
    "'Tuple[bytes]'",
    "'Tuple[unicode]'",
    "'Tuple[str] non-ascii'",
]


text_cases = [
    # XXX: try byte strings

    "",             # empty
    "Loading",      # ascii str
    "ℙƴ☂ℌøἤ",       # non-ascii in str
    u"Загрузка",    # non-ascii in unicode str
]


text_ids = [
    "'empty'",
    "'ascii str'",
    "'non-ascii str'",
    "'non-ascii unicode str'",
]


@pytest.fixture(scope="session")
def interval():
    return 80


@pytest.fixture(scope="session", params=frame_cases, ids=frame_ids)
def frames(request):
    return request.param


@pytest.fixture(scope="session", params=text_cases, ids=text_ids)
def text(request):
    return request.param


@pytest.fixture(
    scope="session",
    params=[False, True],
    ids=["'left'", "'right'"]
)
def right(request):
    return request.param


@pytest.fixture(
    scope="session",
    params=[False, True],
    ids=["default", "reverse"]
)
def reverse(request):
    return request.param


def color_id_func(case):
    if isinstance(case, tuple):
        color, _ = case
    else:
        color = case

    if not color or callable(color):
        val = repr(color)
    else:
        val = color

    return val


@pytest.fixture(scope="session", ids=color_id_func, params=[
    # Empty values
    ("", ""),
    (None, None),

    # Supported text colors
    ("red", "red"),
    ("green", "green"),
    ("yellow", "yellow"),
    ("blue", "blue"),
    ("magenta", "magenta"),
    ("cyan", "cyan"),
    ("white", "white"),

    # Unsupported text colors
    ("black", ValueError()),
    ("brown", ValueError()),
    ("orange", ValueError()),

    # Uppercase handling
    ("Red", "red"),
    ("grEEn", "green"),
    ("BlacK", ValueError()),

    # Callables
    (
        lambda frame: colored(frame, 'red', attrs=['bold']),
        lambda frame: colored(frame, 'red', attrs=['bold']),
    ),
])
def colors_test_cases(request):
    return request.param


@pytest.fixture(scope="session", ids=color_id_func, params=[
    "red",
    "green",
    "yellow",
    "blue",
    "magenta",
    "cyan",
    "white",
    lambda frame: colored(frame, 'red', attrs=['bold']),
])
def supported_colors(request):
    return request.param


@pytest.fixture(scope="session", params=[
    # Empty
    b"",
    u"",

    # Success
    b"OK",
    u"OK",
    b"\xe2\x9c\x94",
    u"✔",

    # Sun
    b"\xe2\x98\x80\xef\xb8\x8f",
    u"☀️",

    # Spark
    b"\xf0\x9f\x92\xa5",
    u"💥",
])
def final_text(request):
    return request.param



# -*- coding: utf-8 -*-

"""
examples.cli_spinners
~~~~~~~~~~~~~~~~~~~~~

Run all spinners from cli-spinners library:
https://github.com/sindresorhus/cli-spinners

Spinners are sorted alphabetically.
"""

import time

from yaspin import yaspin
from yaspin.spinners import Spinners


def main():
    names = [
        attr for attr in dir(Spinners)
        if not callable(getattr(Spinners, attr))
        if not attr.startswith("_")
    ]
    for name in sorted(names):
        spinner = getattr(Spinners, name)
        with yaspin(spinner, text=name):
            time.sleep(3)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from __future__ import absolute_import

from .__version__ import __version__  # noqa
from .base_spinner import Spinner
from .yaspin import yaspin


__all__ = ('yaspin', 'Spinner',)



# -*- coding: utf-8 -*-

"""
examples.hide_show
~~~~~~~~~~~~~~~~~~

Basic usage of the ``hide`` and ``show`` methods.
"""

import sys
import time

from yaspin import yaspin


def main():
    with yaspin(text='Downloading images') as sp:
        # task 1
        time.sleep(1)
        sp.hide()
        sys.stdout.write('> image 1 download complete\n')
        sp.show()

        # task 2
        time.sleep(2)
        sp.hide()
        sys.stdout.write('> image 2 download complete\n')
        sp.show()

        # finalize
        sp.ok()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""
examples.hide_show_prompt_toolkit
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Usage of the ``hide`` and ``show`` methods with prompt_toolkit's printing.

Requires python-prompt-tooklit (2.0):
https://github.com/jonathanslenders/python-prompt-toolkit/

.. code:: bash
    pip install -e git+https://github.com/jonathanslenders/python-prompt-toolkit@2.0#egg=python-prompt-toolkit
"""

from __future__ import print_function

import sys
import time

from yaspin import yaspin

try:
    from prompt_toolkit import HTML, print_formatted_text
    from prompt_toolkit.styles import Style
except ImportError:
    print(
        "This example requires python-prompt-tooklit >= 2.0:\n"
        "https://github.com/jonathanslenders/python-prompt-toolkit/tree/2.0\n"
        "\nTo install it, run:\n"
        "pip install -e git+https://github.com/jonathanslenders/python-prompt-toolkit@2.0#egg=python-prompt-toolkit"
    )
    sys.exit(1)


# override print with feature-rich ``print_formatted_text`` from prompt_toolkit
print = print_formatted_text

# build a basic prompt_toolkit style for styling the HTML wrapped text
style = Style.from_dict({
    'msg': '#4caf50 bold',
    'sub-msg': '#616161 italic'
})


with yaspin(text='Downloading images') as sp:

    # task 1
    time.sleep(1)
    sp.hide()
    print(HTML(
        u'<b>></b> <msg>image 1</msg> <sub-msg>download complete</sub-msg>'
    ), style=style)
    sp.show()

    # task 2
    time.sleep(2)
    sp.hide()
    print(HTML(
        u'<b>></b> <msg>image 2</msg> <sub-msg>download complete</sub-msg>'
    ), style=style)
    sp.show()

    # finalize
    sp.ok()



# -*- coding: utf-8 -*-

"""
yaspin.compat
~~~~~~~~~~~~~

Compatibility layer.
"""

import sys


PY2 = sys.version_info[0] == 2


if PY2:
    builtin_str = str
    bytes = str
    str = unicode                # noqa
    basestring = basestring      # noqa

    def iteritems(dct):
        return dct.iteritems()

else:
    builtin_str = str
    bytes = bytes
    str = str
    basestring = (str, bytes)

    def iteritems(dct):
        return dct.items()



# -*- coding: utf-8 -*-

"""
examples.advanced_colors
~~~~~~~~~~~~~~~~~~~~~~~~

Use familiar API of the `termcolor.colored` function to apply any
color, highlight, attribute or their mix to your yaspin spinner.
"""

import time

from yaspin import yaspin
from yaspin.spinners import Spinners
from yaspin.termcolor import colored


def test_highlights():
    test_items = [
        (
            "On gray color",
            lambda frame: colored(frame, on_color='on_grey')
        ),
        (
            "On red color",
            lambda frame: colored(frame, on_color='on_red')
        ),
        (
            "On green color",
            lambda frame: colored(frame, on_color='on_green')
        ),
        (
            "On yellow color",
            lambda frame: colored(frame, on_color='on_yellow')
        ),
        (
            "On blue color",
            lambda frame: colored(frame, on_color='on_blue')
        ),
        (
            "On magenta color",
            lambda frame: colored(frame, on_color='on_magenta')
        ),
        (
            "On cyan color",
            lambda frame: colored(frame, on_color='on_cyan')
        ),
        (
            "On white color",
            lambda frame: colored(frame, on_color='on_white')
        ),
    ]
    for text, color_func in test_items:
        with yaspin(Spinners.bouncingBall, text=text, color=color_func):
            time.sleep(2)


def test_attributes():
    test_items = [
        (
            "Bold gray color",
            lambda frame: colored(frame, 'grey', attrs=['bold'])
        ),
        (
            "Dark red color",
            lambda frame: colored(frame, 'red', attrs=['dark'])
        ),
        (
            "Underline green color",
            lambda frame: colored(frame, 'green', attrs=['underline'])
        ),
        (
            "Blink yellow color",
            lambda frame: colored(frame, 'yellow', attrs=['blink'])
        ),
        (
            "Reversed blue color",
            lambda frame: colored(frame, 'blue', attrs=['reverse'])
        ),
        (
            "Concealed magenta color",
            lambda frame: colored(frame, 'magenta', attrs=['concealed'])
        ),
        (
            "Bold underline reverse cyan color",
            lambda frame: colored(frame, 'cyan', attrs=['bold', 'underline', 'reverse'])  # noqa
        ),
        (
            "Dark blink concealed white color",
            lambda frame: colored(frame, 'white', attrs=['dark', 'blink', 'concealed'])  # noqa
        ),
    ]
    for text, color_func in test_items:
        with yaspin(Spinners.bouncingBar, text=text, color=color_func):
            time.sleep(2)


def test_mixing():
    test_items = [
        (
            "Underline red on grey color",
            lambda frame: colored(frame, 'red', 'on_grey', attrs=['underline'])
        ),
        (
            "Reversed green on red color",
            lambda frame: colored(frame, 'green', 'on_red', attrs=['reverse'])
        ),
    ]
    for text, color_func in test_items:
        with yaspin(Spinners.pong, text=text, color=color_func):
            time.sleep(2)


def main():
    test_highlights()
    test_attributes()
    test_mixing()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""
examples.colors
~~~~~~~~~~~~~~~

Basic usage of the color argument.
"""

import time

from yaspin import yaspin


def all_text_colors():
    with yaspin(text="Colors!") as sp:
        time.sleep(2)

        colors = ("red", "green", "yellow", "blue", "magenta", "cyan", "white")
        for color in colors:
            sp.color = color
            sp.text = color
            time.sleep(2)


def colored_finalizer():
    with yaspin(text="Processing...", color="yellow") as sp:
        time.sleep(2)
        sp.ok('☀️')


@yaspin(text="Colored decorator", color="yellow")
def decorated_function():
    time.sleep(2)


def main():
    all_text_colors()
    colored_finalizer()
    decorated_function()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""
examples.write_method
~~~~~~~~~~~~~~~~~~~~~

Basic usage of ``write`` method.
"""

import time

from yaspin import yaspin


def main():
    with yaspin(text='Downloading images') as sp:
        # task 1
        time.sleep(1)
        sp.write('> image 1 download complete')

        # task 2
        time.sleep(2)
        sp.write('> image 2 download complete')

        # finalize
        sp.ok()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""
examples.demo
~~~~~~~~~~~~~

Yaspin features demonstration.
"""

import random
import time

from yaspin import yaspin
from yaspin.spinners import Spinners
from yaspin.termcolor import colored


def any_spinner_you_like():
    params = [
        ("line",         0.8),
        ("dots10",       0.8),
        ("dots11",       0.8),
        ("simpleDots",   1),
        ("star",         0.5),
        ("balloon",      0.7),
        ("noise",        0.5),
        ("arc",          0.5),
        ("arrow2",       0.5),
        ("bouncingBar",  1),
        ("bouncingBall", 1),
        ("smiley",       0.7),
        ("hearts",       0.5),
        ("clock",        0.7),
        ("earth",        0.7),
        ("moon",         0.7),
        ("runner",       0.5),
        ("pong",         1),
        ("shark",        1.5),
        ("christmas",    0.5),
    ]
    random.shuffle(params)

    # Setup printing
    max_len = 45
    msg = "[Any spinner you like]"

    for name, period in params:
        spinner = getattr(Spinners, name)

        spaces_qty = max_len - len(name) - len(msg) - len(spinner.frames[0])
        text = "{0}{1}{2}".format(name, " " * spaces_qty, msg)

        with yaspin(spinner, text=text, color="cyan"):
            time.sleep(period)


def colors_simple():
    # Setup printing
    max_len = 25
    msg = "[Colors]"

    with yaspin(Spinners.dots12) as sp:
        colors = ("red", "green", "yellow", "blue", "magenta", "cyan", "white")

        for color in colors:
            spaces_qty = max_len - len(color) - len(msg)
            text = "{0}{1}{2}".format(color, " " * spaces_qty, msg)

            sp.color = color
            sp.text = text
            time.sleep(0.7)


def color_highlights():
    setups = [
        (
            "On gray color",
            lambda frame: colored(frame, on_color='on_grey')
        ),
        (
            "On red color",
            lambda frame: colored(frame, on_color='on_red')
        ),
        (
            "On green color",
            lambda frame: colored(frame, on_color='on_green')
        ),
        (
            "On yellow color",
            lambda frame: colored(frame, on_color='on_yellow')
        ),
        (
            "On blue color",
            lambda frame: colored(frame, on_color='on_blue')
        ),
        (
            "On magenta color",
            lambda frame: colored(frame, on_color='on_magenta')
        ),
        (
            "On cyan color",
            lambda frame: colored(frame, on_color='on_cyan')
        ),
        (
            "On white color",
            lambda frame: colored(frame, on_color='on_white')
        ),
    ]
    # Setup printing
    max_len = 40
    msg = "[Color Highlights]"

    with yaspin(Spinners.bouncingBall) as sp:

        for name, color_func in setups:
            spaces_qty = max_len - len(name) - len(msg)
            text = "{0}{1}{2}".format(name, " " * spaces_qty, msg)

            sp.color = color_func
            sp.text = text

            time.sleep(0.5)


def color_attributes():
    setups = [
        (
            "Bold gray color",
            lambda frame: colored(frame, 'grey', attrs=['bold'])
        ),
        (
            "Dark red color",
            lambda frame: colored(frame, 'red', attrs=['dark'])
        ),
        (
            "Underline green color",
            lambda frame: colored(frame, 'green', attrs=['underline'])
        ),
        (
            "Blink yellow color",
            lambda frame: colored(frame, 'yellow', attrs=['blink'])
        ),
        (
            "Reversed blue color",
            lambda frame: colored(frame, 'blue', attrs=['reverse'])
        ),
        (
            "Concealed magenta color",
            lambda frame: colored(frame, 'magenta', attrs=['concealed'])
        ),
    ]
    # Setup printing
    max_len = 42
    msg = "[Color Attributes]"

    with yaspin(Spinners.bouncingBall) as sp:

        for name, color_func in setups:
            spaces_qty = max_len - len(name) - len(msg)
            text = "{0}{1}{2}".format(name, " " * spaces_qty, msg)

            sp.color = color_func
            sp.text = text

            time.sleep(0.8)


def color_craziness():
    setups = [
        (
            "Bold underline reverse cyan color",
            lambda frame: colored(frame, 'cyan', attrs=['bold', 'underline', 'reverse'])  # noqa
        ),
        (
            "Dark blink concealed white color",
            lambda frame: colored(frame, 'white', attrs=['dark', 'blink', 'concealed'])  # noqa
        ),
        (
            "Underline red on grey color",
            lambda frame: colored(frame, 'red', 'on_grey', attrs=['underline'])
        ),
        (
            "Reversed green on red color",
            lambda frame: colored(frame, 'green', 'on_red', attrs=['reverse'])
        ),
    ]
    # Setup printing
    max_len = 58
    msg = "[Color Craziness 🙀 ]"

    with yaspin(Spinners.pong) as sp:

        for name, color_func in setups:
            spaces_qty = max_len - len(name) - len(msg)
            text = "{0}{1}{2}".format(name, " " * spaces_qty, msg)

            sp.color = color_func
            sp.text = text

            time.sleep(1.3)


def right_spinner():
    with yaspin(text="Right spinner", right=True, color="cyan") as sp:
        time.sleep(2)

        # Switch to left spinner
        sp.right = False
        sp.text = "Left spinner"

        time.sleep(2)


def reversed_spinner():
    with yaspin(text="Reversed spinner", reverse=True, color="cyan") as sp:
        time.sleep(1)

        sp.spinner = Spinners.line

        time.sleep(1)

        sp.text = "Enjoy!"
        sp.ok('☀️')


def main():
    any_spinner_you_like()
    colors_simple()
    color_highlights()
    color_attributes()
    color_craziness()
    right_spinner()
    reversed_spinner()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

"""
yaspin.yaspin
~~~~~~~~~~~~~

A lightweight terminal spinner.
"""

from __future__ import absolute_import

import functools
import itertools
import sys
import threading
import time

from .base_spinner import default_spinner
from .compat import PY2, basestring, builtin_str, bytes, str
from .constants import ENCODING
from .helpers import to_unicode
from .termcolor import colored


class Yaspin(object):
    """Implements a context manager that spawns a thread
    to write spinner frames into a tty (stdout) during
    context execution.
    """

    # When Python finds its output attached to a terminal,
    # it sets the sys.stdout.encoding attribute to the terminal's encoding.
    # The print statement's handler will automatically encode unicode
    # arguments into bytes.
    #
    # In Py2 when piping or redirecting output, Python does not detect
    # the desired character set of the output, it sets sys.stdout.encoding
    # to None, and print will invoke the default "ascii" codec.
    #
    # Py3 invokes "UTF-8" codec by default.
    #
    # Thats why in Py2, output should be encoded manually with desired
    # encoding in order to support pipes and redirects.

    def __init__(self, spinner=None, text='',
                 color=None, right=False, reverse=False):
        self._spinner = self._set_spinner(spinner)
        self._frames = self._set_frames(self._spinner, reverse)
        self._interval = self._set_interval(self._spinner)
        self._cycle = self._set_cycle(self._frames)
        self._text = self._set_text(text)
        self._color = self._set_color(color) if color else color

        self._right = right
        self._reverse = reverse

        self._stop_spin = None
        self._hide_spin = None
        self._spin_thread = None
        self._last_frame = None

    #
    # Dunders
    #
    def __repr__(self):
        repr_ = u'<Yaspin frames={0!s}>'.format(self._frames)
        if PY2:
            return repr_.encode(ENCODING)
        return repr_

    def __enter__(self):
        self.start()
        return self

    def __exit__(self, exc_type, exc_val, traceback):
        # Avoid stop() execution for the 2nd time
        if self._spin_thread.is_alive():
            self.stop()
        return False  # nothing is handled

    def __call__(self, fn):
        @functools.wraps(fn)
        def inner(*args, **kwargs):
            with self:
                return fn(*args, **kwargs)
        return inner

    #
    # Properties
    #
    @property
    def spinner(self):
        return self._spinner

    @spinner.setter
    def spinner(self, sp):
        self._spinner = self._set_spinner(sp)
        self._frames = self._set_frames(self._spinner, self._reverse)
        self._interval = self._set_interval(self._spinner)
        self._cycle = self._set_cycle(self._frames)

    @property
    def text(self):
        return self._text

    @text.setter
    def text(self, txt):
        self._text = self._set_text(txt)

    @property
    def color(self):
        return self._color

    @color.setter
    def color(self, value):
        self._color = self._set_color(value) if value else value

    @property
    def right(self):
        return self._right

    @right.setter
    def right(self, value):
        self._right = value

    @property
    def reverse(self):
        return self._reverse

    @reverse.setter
    def reverse(self, value):
        self._reverse = value
        self._frames = self._set_frames(self._spinner, self._reverse)
        self._cycle = self._set_cycle(self._frames)

    #
    # Public
    #
    def start(self):
        if sys.stdout.isatty():
            self._hide_cursor()

        self._stop_spin = threading.Event()
        self._hide_spin = threading.Event()
        self._spin_thread = threading.Thread(target=self._spin)
        self._spin_thread.start()

    def stop(self):
        if self._spin_thread:
            self._stop_spin.set()
            self._spin_thread.join()

        sys.stdout.write("\r")
        self._clear_line()

        if sys.stdout.isatty():
            self._show_cursor()

    def hide(self):
        """Hide the spinner to allow for custom writing to the terminal."""
        thr_is_alive = self._spin_thread and self._spin_thread.is_alive()

        if thr_is_alive and not self._hide_spin.is_set():
            # set the hidden spinner flag
            self._hide_spin.set()

            # clear the current line
            sys.stdout.write("\r")
            self._clear_line()

            # flush the stdout buffer so the current line can be rewritten to
            sys.stdout.flush()

    def show(self):
        """Show the hidden spinner."""
        thr_is_alive = self._spin_thread and self._spin_thread.is_alive()

        if thr_is_alive and self._hide_spin.is_set():
            # clear the hidden spinner flag
            self._hide_spin.clear()

            # clear the current line so the spinner is not appended to it
            sys.stdout.write("\r")
            self._clear_line()

    def write(self, text):
        """Write text in the terminal without breaking the spinner."""
        # similar to tqdm.write()
        # https://pypi.python.org/pypi/tqdm#writing-messages
        sys.stdout.write("\r")
        self._clear_line()

        _text = to_unicode(text)
        if PY2:
            _text = _text.encode(ENCODING)

        # Ensure output is bytes for Py2 and Unicode for Py3
        assert isinstance(_text, builtin_str)

        sys.stdout.write("{0}\n".format(_text))

    def ok(self, text="OK"):
        """Set Ok (success) finalizer to a spinner."""
        _text = text if text else "OK"
        self._freeze(_text)

    def fail(self, text="FAIL"):
        """Set fail finalizer to a spinner."""
        _text = text if text else "FAIL"
        self._freeze(_text)

    #
    # Protected
    #
    def _freeze(self, final_text):
        """Stop spinner, compose last frame and 'freeze' it."""
        text = to_unicode(final_text).strip()
        self._last_frame = self._compose_out(text, mode="last")

        # Should be stopped here, otherwise prints after
        # self._freeze call will mess up the spinner
        self.stop()
        sys.stdout.write(self._last_frame)

    def _spin(self):
        while not self._stop_spin.is_set():

            if self._hide_spin.is_set():
                # Wait a bit to avoid wasting cycles
                time.sleep(self._interval)
                continue

            # Compose output
            spin_phase = next(self._cycle)
            out = self._compose_out(spin_phase)

            # Write
            sys.stdout.write(out)
            self._clear_line()
            sys.stdout.flush()

            # Wait
            time.sleep(self._interval)
            sys.stdout.write('\b')

    def _compose_out(self, frame, mode=None):
        # Ensure Unicode input
        assert isinstance(frame, str)
        assert isinstance(self._text, str)

        frame = frame.encode(ENCODING) if PY2 else frame
        text = self._text.encode(ENCODING) if PY2 else self._text

        if self._color and callable(self._color):
            color_fn = self._color
            frame = color_fn(frame)
        if self._color and not callable(self._color):
            frame = colored(frame, self._color)

        if self._right:
            frame, text = text, frame

        if not mode:
            out = "\r{0} {1}".format(frame, text)
        else:
            out = "{0} {1}\n".format(frame, text)

        # Ensure output is bytes for Py2 and Unicode for Py3
        assert isinstance(out, builtin_str)

        return out

    #
    # Static
    #
    @staticmethod
    def _set_spinner(spinner):
        if not spinner:
            sp = default_spinner

        if hasattr(spinner, "frames") and hasattr(spinner, "interval"):
            if not spinner.frames or not spinner.interval:
                sp = default_spinner
            else:
                sp = spinner
        else:
            sp = default_spinner

        return sp

    @staticmethod
    def _set_frames(spinner, reverse):
        # type: (base_spinner.Spinner, bool) -> Union[str, List]
        uframes = None      # unicode frames
        uframes_seq = None  # sequence of unicode frames

        if isinstance(spinner.frames, basestring):
            uframes = to_unicode(spinner.frames) if PY2 else spinner.frames

        # TODO (pavdmyt): support any type that implements iterable
        if isinstance(spinner.frames, (list, tuple)):

            # Empty spinner.frames is handled by Yaspin._set_spinner
            if spinner.frames and isinstance(spinner.frames[0], bytes):
                uframes_seq = [to_unicode(frame) for frame in spinner.frames]
            else:
                uframes_seq = spinner.frames

        _frames = uframes or uframes_seq
        if not _frames:
            raise ValueError(
                "{0!r}: no frames found in spinner".format(spinner)
            )

        # Builtin reversed returns reverse iterator,
        # which adds unnecessary difficulty for returning
        # unicode value;
        # Hence using [::-1] syntax
        frames = _frames[::-1] if reverse else _frames

        return frames

    @staticmethod
    def _set_interval(spinner):
        # Milliseconds to Seconds
        return spinner.interval * 0.001

    @staticmethod
    def _set_cycle(frames):
        return itertools.cycle(frames)

    @staticmethod
    def _set_text(text):
        if PY2:
            return to_unicode(text)
        return text

    @staticmethod
    def _set_color(color):

        if callable(color):
            return color

        available_text_colors = (
            "red", "green", "yellow", "blue", "magenta", "cyan", "white",
        )

        c_lower = color.lower()

        if c_lower not in available_text_colors:
            raise ValueError(
                "{0}: unsupported text color. Use one of the: {1}"
                .format(c_lower, available_text_colors)
            )

        return c_lower

    @staticmethod
    def _hide_cursor():
        sys.stdout.write("\033[?25l")
        sys.stdout.flush()

    @staticmethod
    def _show_cursor():
        sys.stdout.write("\033[?25h")
        sys.stdout.flush()

    @staticmethod
    def _clear_line():
        sys.stdout.write("\033[K")


def yaspin(spinner=None, text='', color=None, right=False, reverse=False):
    """Display spinner in stdout.

    Can be used as a context manager or as a function decorator.

    Arguments:
        spinner (yaspin.Spinner, optional): Spinner to use.
        text (str, optional): Text to show along with spinner.
        color (str, callable, optional): Color or color style of the spinner.
        right (bool, optional): Place spinner to the right end
            of the text string.
        reverse (bool, optional): Reverse spin direction.

    Returns:
        yaspin.Yaspin: instance of the Yaspin class.

    Raises:
        ValueError: If unsupported `color` is specified.

    Example::

        # Use as a context manager
        with yaspin():
            some_operations()

        # Context manager with text
        with yaspin(text="Processing..."):
            some_operations()

        # Context manager with custom sequence
        with yaspin(Spinner('-\\|/', 150)):
            some_operations()

        # As decorator
        @yaspin(text="Loading...")
        def foo():
            time.sleep(5)

        foo()

    """
    return Yaspin(
        spinner=spinner,
        text=text,
        color=color,
        right=right,
        reverse=reverse,
    )



# -*- coding: utf-8 -*-

"""
tests.test_yaspin
~~~~~~~~~~~~~~~~~

Basic unittests.
"""

from __future__ import absolute_import

from collections import namedtuple

import pytest

from yaspin import Spinner, yaspin
from yaspin.base_spinner import default_spinner


@pytest.mark.parametrize("spinner, expected", [
    # None
    (None, default_spinner),

    # hasattr(spinner, "frames") and not hasattr(spinner, "interval")
    (namedtuple('Spinner', "frames")("-\\|/"), default_spinner),

    # not hasattr(spinner, "frames") and hasattr(spinner, "interval")
    (namedtuple('Spinner', "interval")(42), default_spinner),

    # Both attrs, not set
    (Spinner("", 0), default_spinner),

    # Both attrs, not frames
    (Spinner("", 42), default_spinner),

    # Both attrs, not interval
    (Spinner("-\\|/", 0), default_spinner),

    # Both attrs, are set
    (Spinner("-\\|/", 42), Spinner("-\\|/", 42)),
])
def test_set_spinner(spinner, expected):
    swirl = yaspin(spinner)
    assert swirl.spinner == expected



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
yaspin
~~~~~~

A lightweight and easy to use terminal spinner.
No external dependencies.

:copyright: (c) 2018 Pavlo Dmytrenko
:license: MIT
"""

import codecs
import os
import sys

from setuptools import find_packages, setup
from setuptools.command.test import test as TestCommand


# Package meta-data
NAME = 'yaspin'
LICENSE = 'MIT'
DESCRIPTION = 'Yet Another Terminal Spinner'
URL = 'https://github.com/pavdmyt/yaspin'
EMAIL = 'mail@pavdmyt.com'
AUTHOR = 'Pavlo Dmytrenko'


def read_version(package):
    version_path = os.path.join(package, '__version__.py')
    with open(version_path, 'r') as fd:
        for line in fd:
            if line.startswith('__version__ = '):
                return line.split()[-1].strip().strip("'")


version = read_version(NAME)


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = ['tests/']
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)


with codecs.open('README.rst', encoding='utf-8') as fd:
    readme = fd.read()


tests_require = [
    "pytest",
]


setup(
    name=NAME,
    version=version,
    author=AUTHOR,
    author_email=EMAIL,
    license=LICENSE,
    description=DESCRIPTION,
    long_description=readme,
    url=URL,
    download_url='https://github.com/pavdmyt/yaspin/archive/master.zip',
    packages=find_packages(exclude=('tests', 'docs', 'examples')),
    include_package_data=True,
    tests_require=tests_require,
    cmdclass={'test': PyTest},
    keywords='progressmeter progress meter rate console terminal console cli'
             ' loading loader indicator spinner spinners time busy wait idle',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: MacOS X',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Other Audience',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Software Development :: User Interfaces',
        'Topic :: System :: Logging',
        'Topic :: System :: Monitoring',
        'Topic :: System :: Shells',
        'Topic :: Terminals',
        'Topic :: Utilities'
    ],
)
